/*******************************

 Rue's anti-fault monitor.


*******************************/

#include "defines.h"
#include "serial.h"
#include "flash.h"

/* Uncomment the following to save code space */
//#define REMOVE_FUSE_AND_LOCK_BIT_SUPPORT
//#define REMOVE_BLOCK_SUPPORT
//#define REMOVE_EEPROM_BYTE_SUPPORT
//#define REMOVE_FLASH_BYTE_SUPPORT


#ifndef REMOVE_BLOCK_SUPPORT
    unsigned char BlockLoad(unsigned int size, unsigned char mem, long *address);
    void BlockRead(unsigned int size, unsigned char mem, long *address);
    
    /* BLOCKSIZE should be chosen so that the following holds: BLOCKSIZE*n = PAGESIZE,  where n=1,2,3... */
    #define BLOCKSIZE PAGESIZE
#endif



void main(void) {
    long          address;
    unsigned char val;
    int           temp_int;

    initbootuart(); // Initialize UART.

    while(1) {
    
        val = recchar(); // Wait for command character.

        if(0) {
        
        } else if(val == CMD_SETADDRESS) { // Set address in words
            sendchar('a') ;
            address = getword();         
            printword( address );    
            sendchar('\r'); // Send OK back.
        
        } else if (val == CMD_GETADDRESS) { // Return address        
            printword( address );
            sendchar('\r'); // Send OK back.      
                
        } else if(val == CMD_CHIPERASE) { // Chip erase...
            for(address = 0; address < APP_END;address += PAGESIZE) {                                     
                PAGE_ERASE(address);  
                WAIT_FOR_SPM();          
            }   
            ENABLE_RWW_SECTION();         
            sendchar('\r'); // Send OK back.
                        
        } else if(val==CMD_GETBLOCKSIZE) {  // Report block size
			printword(BLOCKSIZE);
			
		} else if(val==CMD_BLOCKWRITE) { // Start block load.
            sendchar('S');
    	    temp_int = getword(); // Get block size.
	    	val = recchar(); // Get memtype.
		    sendchar( BlockLoad(temp_int,val,&address) ); // Block load.
		    
	    } else if(val==CMD_BLOCKREAD) { // Start block read.
	        sendchar('S');
    	    temp_int = getword(); // Get block size.
			val = recchar(); // Get memtype
    		BlockRead(temp_int,val,&address); // Block read
    		
		} else if(val==CMD_GETFLASH) { // Read program memory.
            WAIT_FOR_SPM();
            ENABLE_RWW_SECTION();
            // address as a word
            printbyte( LOAD_PROGRAM_MEMORY( (address << 1)+0 ) );
            printbyte( LOAD_PROGRAM_MEMORY( (address << 1)+1 ) );
            address++; 
            
        } else if(val==CMD_ADDWORD) { // add byte to memory programing buffer
            sendchar('D');
            temp_int = getword(); // Get and insert high byte.
            WAIT_FOR_SPM();
            FILL_TEMP_WORD( (address << 1), temp_int ); // Convert word-address to byte-address and fill.
            address++; // Auto-advance to next Flash word.
            sendchar('\r'); // Send OK back.
            
        } else if(val== CMD_COMMITFLASH)  { // Write page.
            if( address >= (APP_END>>1) ) { // Protect bootloader area.
                sendchar('?');
            } else  {
                WAIT_FOR_SPM();
                PAGE_WRITE( address << 1 ); // Convert word-address to byte-address and write.
                ENABLE_RWW_SECTION();
            }
            sendchar('\r'); // Send OK back.
            
        } else if (val == CMD_SETEEPROM)  { // Write EEPROM memory.
            WAIT_FOR_SPM();
            EEARL = address; // Setup EEPROM address.
            EEARH = (address >> 8);
            EEDR = getbyte(); // Get byte.
            EECR |= (1<<EEMWE); // Write byte.
            EECR |= (1<<EEWE);
            while (EECR & (1<<EEWE)); // Wait for write operation to finish.

            address++; // Auto-advance to next EEPROM byte.
            sendchar('\r');// Send OK back.
            
        } else if (val == CMD_GETEEPROM) { // Read EEPROM memory.
            EEARL = address; // Setup EEPROM address.
            EEARH = (address >> 8);
            EECR |= (1<<EERE); // Read byte...
            printbyte(EEDR); // ...and send it back.
            address++; // Auto-advance to next EEPROM byte.
            
        }  else if(val==CMD_SETLOCKS)  { // Write lockbits.
            WAIT_FOR_SPM();
            _SET_LOCK_BITS( recchar() ); // Read and set lock bits.
            sendchar('\r'); // Send OK back.
            
        }  else if(val==CMD_GETLOCKS) { // Read lock bits.
            WAIT_FOR_SPM();
            printbyte( _GET_LOCK_BITS() );
            
        } else if(val==CMD_GETFUSESL) { // Read fuse bits.
            WAIT_FOR_SPM();
            printbyte( _GET_LOW_FUSES() );
            
        } else if(val==CMD_GETFUSESH)   { // Read high fuse bits.
            WAIT_FOR_SPM();
            printbyte( _GET_HIGH_FUSES() );
            
        } else if(val==CMD_GETFUSESX)   { // Read extended fuse bits.
            WAIT_FOR_SPM();
            printbyte( _GET_EXTENDED_FUSES() );
            
        } else if(val==CMD_EXIT)   { // Exit bootloader.
            WAIT_FOR_SPM();
            ENABLE_RWW_SECTION();
            sendchar('\r');
            
        } else if(val!=0x1b) {
            sendchar('?');
            
        }        
    } // end of  for(;;)
   
} // end of main


#ifndef REMOVE_BLOCK_SUPPORT
unsigned char BlockLoad(unsigned int size, unsigned char mem, long *address)
{
    unsigned char buffer[BLOCKSIZE];
    unsigned int data;
    long tempaddress;
	
    // EEPROM memory type.
    if(mem=='E')
    {
        /* Fill buffer first, as EEPROM is too slow to copy with UART speed */
        for(tempaddress=0;tempaddress<size;tempaddress++)
            buffer[tempaddress] = recchar();

        /* Then program the EEPROM */
        WAIT_FOR_SPM();
    	for( tempaddress=0; tempaddress < size; tempaddress++)
    	{
	        EEARL = *address; // Setup EEPROM address
            EEARH = ((*address) >> 8);
            EEDR = buffer[tempaddress]; // Get byte.
            EECR |= (1<<EEMWE); // Write byte.
            EECR |= (1<<EEWE);
            while (EECR & (1<<EEWE)); // Wait for write operation to finish.

  			(*address)++; // Select next EEPROM byte
        }

        return '\r'; // Report programming OK
    } else if(mem=='F')  {     // Flash memory.
        (*address) <<= 1; // Convert address to bytes temporarily.
        tempaddress = (*address);  // Store address in page.
	
        do {
            data = getword();
            FILL_TEMP_WORD(*address, data);
            (*address)+=2; // Select next word in memory.
            size -= 2; // Reduce number of bytes to write by two.
        } while(size); // Loop until all bytes written.

	PAGE_WRITE(tempaddress);
	WAIT_FOR_SPM();
	ENABLE_RWW_SECTION();
        (*address) >>= 1; // Convert address back to Flash words again.
        return '\r'; // Report programming OK
    } else {
        return '?';
    }
}


void BlockRead(unsigned int size, unsigned char mem, long *address)
{
    // EEPROM memory type.
    if (mem=='E')  {// Read EEPROM
        do {
            EEARL = *address; // Setup EEPROM address
            EEARH = ((*address) >> 8);
            (*address)++; // Select next EEPROM byte
            EECR |= (1<<EERE); // Read EEPROM
            printbyte(EEDR); // Transmit EEPROM dat ato PC

            size--; // Decrease number of bytes to read
        } while (size); // Repeat until all block has been read
        
    } else if(mem=='F')	{ // Flash memory
        (*address) <<= 1; // Convert address to bytes temporarily.
        do {
            printbyte( LOAD_PROGRAM_MEMORY(*address) );
            printbyte( LOAD_PROGRAM_MEMORY((*address)+1) );
            (*address) += 2; // Select next word in memory.
            size -= 2; // Subtract two bytes from number of bytes to read
        } while (size); // Repeat until all block has been read
        (*address) >>= 1; // Convert address back to Flash words again.
    }
}
#endif


/* end of file */
