/*******************************

 Rue's anti-fault monitor.

*******************************/

#include "defines.h"
#include "serial.h"
#include "flash.h"

/* Uncomment the following to save code space */
//#define REMOVE_FUSE_AND_LOCK_BIT_SUPPORT
//#define REMOVE_BLOCK_SUPPORT
//#define REMOVE_EEPROM_BYTE_SUPPORT
//#define REMOVE_FLASH_BYTE_SUPPORT


#ifndef REMOVE_BLOCK_SUPPORT
    unsigned char BlockLoad(unsigned int size, unsigned char mem, long *address);
    void BlockRead(unsigned int size, unsigned char mem, long *address);
    
    /* BLOCKSIZE should be chosen so that the following holds: BLOCKSIZE*n = PAGESIZE,  where n=1,2,3... */
    #define BLOCKSIZE PAGESIZE
#endif



void main(void) {
    long          address;
    unsigned char val;
    int           temp_int;

    initbootuart(); // Initialize UART.

    while(1) {
    
        val = recchar(); // Wait for command character.

        if(0) {
        
        } else if(val=='A') { // Set address...
         // NOTE: Flash addresses are given in words, not bytes.
            //address=(recchar()<<8) | recchar(); // Read address high and low byte.
            address = getword();         
            printword( address );    
            sendchar('\r'); // Send OK back.
        
        } else if (val=='a') { // Return address
        
        printword( address );
        sendchar('\r'); // Send OK back.      
                
        } else if(val=='e') { // Chip erase...
            for(address = 0; address < APP_END;address += PAGESIZE)
            {                 
                WAIT_FOR_SPM();    
                boot_page_erase(address);            
            }
            
            sendchar('\r'); // Send OK back.
        }

#ifndef REMOVE_BLOCK_SUPPORT
        
        else if(val=='b') {  // Check block load support.
			sendchar('Y'); // Report block load supported.
			printbyte((BLOCKSIZE>>8) & 0xFF); // MSB first.
			printbyte(BLOCKSIZE&0xFF); // Report BLOCKSIZE (bytes).
		}
        
		else if(val=='B') { // Start block load.
    	    temp_int = getword(); // Get block size.
	    	val = recchar(); // Get memtype.
		    sendchar( BlockLoad(temp_int,val,&address) ); // Block load.
		    
	    } else if(val=='g') { // Start block read.
    	    temp_int = getword(); // Get block size.
			val = recchar(); // Get memtype
    		BlockRead(temp_int,val,&address); // Block read
		}		
#endif

#ifndef REMOVE_FLASH_BYTE_SUPPORT
        
        else if(val=='R') { // Read program memory.
            WAIT_FOR_SPM();
            _ENABLE_RWW_SECTION();
            // address as a word
            printbyte( LOAD_PROGRAM_MEMORY( (address << 1)+0 ) );
            printbyte( LOAD_PROGRAM_MEMORY( (address << 1)+1 ) );

            address++; 
        }
        
        // Write program memory, high byte.
        else if(val=='C') {
            temp_int = getword(); // Get and insert high byte.
            WAIT_FOR_SPM();
            FILL_TEMP_WORD( (address << 1), temp_int ); // Convert word-address to byte-address and fill.
            address++; // Auto-advance to next Flash word.
            sendchar('\r'); // Send OK back.
        }
        
        else if(val== 'm')  { // Write page.
            if( address >= (APP_END>>1) ) { // Protect bootloader area.
                sendchar('?');
            } else  {
                WAIT_FOR_SPM();
                PAGE_WRITE( address << 1 ); // Convert word-address to byte-address and write.
            }

            sendchar('\r'); // Send OK back.
        }
#endif

#ifndef REMOVE_EEPROM_BYTE_SUPPORT
        
        else if (val == 'D')  { // Write EEPROM memory.
            WAIT_FOR_SPM();
            EEARL = address; // Setup EEPROM address.
            EEARH = (address >> 8);
            EEDR = getbyte(); // Get byte.
            EECR |= (1<<EEMWE); // Write byte.
            EECR |= (1<<EEWE);
            while (EECR & (1<<EEWE)); // Wait for write operation to finish.

            address++; // Auto-advance to next EEPROM byte.
            sendchar('\r');// Send OK back.
        }
        
        else if (val == 'd') { // Read EEPROM memory.
            EEARL = address; // Setup EEPROM address.
            EEARH = (address >> 8);
            EECR |= (1<<EERE); // Read byte...
            printbyte(EEDR); // ...and send it back.
            address++; // Auto-advance to next EEPROM byte.
        }
#endif

#ifndef REMOVE_FUSE_AND_LOCK_BIT_SUPPORT        
        else if(val=='l')  { // Write lockbits.
            WAIT_FOR_SPM();
            _SET_LOCK_BITS( recchar() ); // Read and set lock bits.
            sendchar('\r'); // Send OK back.
        }
        
        else if(val=='r') { // Read lock bits.
            WAIT_FOR_SPM();
            printbyte( _GET_LOCK_BITS() );
        }
        
        else if(val=='F') { // Read fuse bits.
            WAIT_FOR_SPM();
            printbyte( _GET_LOW_FUSES() );
        }
        
        else if(val=='N')   { // Read high fuse bits.
            WAIT_FOR_SPM();
            printbyte( _GET_HIGH_FUSES() );
        }
        
        else if(val=='Q')   { // Read extended fuse bits.
            WAIT_FOR_SPM();
            printbyte( _GET_EXTENDED_FUSES() );
        }
#endif
        
        else if(val=='E')   { // Exit bootloader.
            WAIT_FOR_SPM();
            _ENABLE_RWW_SECTION();
            sendchar('\r');
        }

        // The last command to accept is ESC (synchronization).
        else if(val!=0x1b)                  // If not ESC, then it is unrecognized...
        {
            sendchar('?');
        }
    } // end of  for(;;)
   
} // end of main


#ifndef REMOVE_BLOCK_SUPPORT
unsigned char BlockLoad(unsigned int size, unsigned char mem, long *address)
{
    unsigned char buffer[BLOCKSIZE];
    unsigned int data;
    long tempaddress;
	
    // EEPROM memory type.
    if(mem=='E')
    {
        /* Fill buffer first, as EEPROM is too slow to copy with UART speed */
        for(tempaddress=0;tempaddress<size;tempaddress++)
            buffer[tempaddress] = recchar();

        /* Then program the EEPROM */
        WAIT_FOR_SPM();
    	for( tempaddress=0; tempaddress < size; tempaddress++)
    	{
	        EEARL = *address; // Setup EEPROM address
            EEARH = ((*address) >> 8);
            EEDR = buffer[tempaddress]; // Get byte.
            EECR |= (1<<EEMWE); // Write byte.
            EECR |= (1<<EEWE);
            while (EECR & (1<<EEWE)); // Wait for write operation to finish.

  			(*address)++; // Select next EEPROM byte
        }

        return '\r'; // Report programming OK
    }

    // Flash memory type.
	else if(mem=='F')
    { // NOTE: For flash programming, 'address' is given in words.
        (*address) <<= 1; // Convert address to bytes temporarily.
        tempaddress = (*address);  // Store address in page.
	
        do {
            data = getword();
            FILL_TEMP_WORD(*address, data);
            (*address)+=2; // Select next word in memory.
            size -= 2; // Reduce number of bytes to write by two.
        } while(size); // Loop until all bytes written.

	PAGE_WRITE(tempaddress);
	WAIT_FOR_SPM();
	_ENABLE_RWW_SECTION();
        (*address) >>= 1; // Convert address back to Flash words again.
        return '\r'; // Report programming OK
    }

    // Invalid memory type?
    else
    {
        return '?';
    }
}


void BlockRead(unsigned int size, unsigned char mem, long *address)
{
    // EEPROM memory type.
    if (mem=='E') // Read EEPROM
    {
        do
        {
            EEARL = *address; // Setup EEPROM address
            EEARH = ((*address) >> 8);
            (*address)++; // Select next EEPROM byte
            EECR |= (1<<EERE); // Read EEPROM
            printbyte(EEDR); // Transmit EEPROM dat ato PC

            size--; // Decrease number of bytes to read
        } while (size); // Repeat until all block has been read
    }

    // Flash memory type.
	else if(mem=='F')
	{
        (*address) <<= 1; // Convert address to bytes temporarily.
        do
        {
            sendchar( LOAD_PROGRAM_MEMORY(*address) );
            sendchar( LOAD_PROGRAM_MEMORY((*address)+1) );
            (*address) += 2; // Select next word in memory.
            size -= 2; // Subtract two bytes from number of bytes to read
        } while (size); // Repeat until all block has been read
        (*address) >>= 1; // Convert address back to Flash words again.
    }
}
#endif


/* end of file */
